"use strict";

exports.__esModule = true;
exports.default = void 0;

var _core = require("@babel/core");

class ImportsCache {
  constructor() {
    this._imports = new WeakMap();
    this._lastImports = new WeakMap();
  }

  store(programPath, url, name, getVal) {
    const key = this._normalizeKey(programPath, url, name);

    const imports = this._ensure(this._imports, programPath);

    if (!imports.has(key)) {
      const {
        node,
        name: id
      } = getVal(programPath.node.sourceType === "script", _core.types.stringLiteral(url), _core.types.identifier(name));
      imports.set(key, id);

      let lastImport = this._lastImports.get(programPath);

      if (lastImport && lastImport.node) {
        lastImport = lastImport.insertAfter(node);
      } else {
        lastImport = programPath.unshiftContainer("body", node);
      }

      lastImport = lastImport[lastImport.length - 1];

      this._lastImports.set(programPath, lastImport);
    }

    return _core.types.identifier(imports.get(key));
  }

  _ensure(map, programPath) {
    if (!map.has(programPath)) map.set(programPath, new Map());
    return map.get(programPath);
  }

  _normalizeKey(programPath, url, name) {
    return `${programPath.node.sourceType}::${url}::${name}`;
  }

}

exports.default = ImportsCache;